/* -*-	Mode:C++ -*- */

/*
 * gtk_blob.hh
 * Copyright (C) 2003 by John Heidemann
 * $Id: gtk_blob.hh,v 1.21 2003/06/21 17:52:40 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef lavaps_gtk_blob_h
#define lavaps_gtk_blob_h

#ifdef USE_GTK_BLOB

#include <gnome.h>
#include <libgnomecanvas/libgnomecanvas.h>

#include "scaled_blob.hh"
#include "const_str.hh"

#include <list>
#include <map>

#include <string>

#define LABEL_BUF_LEN 256  // buffer big enough to contain process names

class gtk_blob_splash {
protected:
	GtkWidget *canvas_w_;

	bool live_;
	GnomeCanvasItem *id_;
	int x_, y_;
	int gray_;
	string text_;
	string text_without_last_;
	string last_msg_;
	int last_msg_count_;

	void revive(const char *msg);
public:
	gtk_blob_splash(GtkWidget *canvas_w) : canvas_w_(canvas_w),
				    live_(false), id_(NULL),
				    x_(50), y_(50), gray_(0),
				    last_msg_count_(0)
		{};
	void splash(const char *msg);
	bool heartbeat();
	void slay();
	void update_wh(int w, int h);
	bool item_is_this_splash(GnomeCanvasItem *i) { return (i != NULL) && (i == id_); };
};

/*
 * structure to keep track of details for the menu callback
 */
class menu_callback_info {
public:
	enum menus { UNDEFINED_MENU, WHO_MENU, WHAT_MENU, HOW_MENU };
	enum modifiers { UNDEFINED_MODIFIER, WHO_ME, WHO_EVERYONE,
			 WHAT_VIRTUAL, WHAT_PHYSICAL, WHAT_BOTH, 
			 HOW_SHRINK, HOW_GROW, HOW_AUTOSIZE, HOW_LOZENGE
	};
	enum menus menu_;
	enum modifiers modifier_;
	const char *desc_;

	menu_callback_info(enum menus menu, enum modifiers modifier, const char *desc) :
		menu_(menu), modifier_(modifier), desc_(desc) {}
};


class gtk_blob : public scaled_blob {
private:
	gtk_blob(const gtk_blob& tb);
	gtk_blob& operator=(const gtk_blob& tb);

protected:
	// constants
	enum { CANVAS_DEFAULT_WIDTH = 100, CANVAS_DEFAULT_HEIGHT = 200 };

	// keep track of blobs, and us in them
	typedef list<gtk_blob*> gtk_blobs_t;
	static gtk_blobs_t gtk_blobs_;
	gtk_blobs_t::iterator me_in_blobs_;
	// and map between items and blobs (many to one)
	static map<GnomeCanvasItem*,gtk_blob*> item_to_blob_map_;

	// a blob is two parts
	GnomeCanvasGroup *group_;  // group for the blob and its shadow
	GnomeCanvasItem *id_, *did_;

	// basic stuff
	static GnomeProgram *program_;
	static GtkWidget *app_;
	static GtkWidget *canvas_w_;

	// user interaction
	static GtkWidget *popup_menu_w_;
	static GtkAccelGroup *popup_accel_group_;
	static gtk_blob *info_blob_hit_;
	static gtk_blob *menu_blob_hit_;
	static int last_x_, last_y_;
	//
	GdkColor fg_color_, bd_color_, dk_color_;
	bool fg_is_whitish_;

	// info window
	static bool mouse_down_;
	static GtkWindow *info_w_;
	static GtkLabel *info_color_label_, *info_text_label_;
	// splash
	static gtk_blob_splash *splash_;

	static void init_menu();
	static void init_canvas();
	static void init_ticker();
	static void init_info();

	void set_point(GnomeCanvasPoints *points, int i, double x, double y);
	GnomeCanvasPoints *set_points(int pct = 100, int offset = 0);

	virtual void redraw();
	static gboolean tick(gpointer data);  // check for processes

	static gtk_blob *find_blob_hit(GtkObject *obj, GdkEvent *event);

	static gboolean popup_menu_event(GtkObject *obj, GdkEventButton *button, gtk_blob *blob_hit);
	static gboolean post_info_event(GtkObject *obj, GdkEventButton *button, gtk_blob *blob_hit);
	void setup_info_for_me();
	static gboolean  blob_event(GtkObject *obj, GdkEvent *event, gpointer data);
	static gboolean configure_event(GtkObject *obj, GdkEvent *event, gpointer data);

	static void shape_window(int w, int h);

public:
	gtk_blob(process_view *pv);
	virtual ~gtk_blob();

	virtual void set_hsb(double h, double s, double b);

	static void splash(const char *msg);
	static void init(int argc, char **argv);
	static void mainloop();

	static void send_signal_callback(GtkWidget *widget, char *signal);
	static void menu_set_callback(GtkWidget *widget, menu_callback_info *signal);
	static void menu_help_callback(GtkWidget *widget, char *signal);
};

#endif /* USE_GTK_BLOB */

#endif /* ! lavaps_gtk_blob_h */
