
/*
 * gtk_compat.cc
 * Copyright (C) 2003 by John Heidemann
 * $Id: gtk_compat.cc,v 1.4 2003/06/21 20:30:27 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * Gnome/gtk functions that IMHO should exist but don't.
 */

#include "config.h"

#ifdef USE_GTK_BLOB

#include <unistd.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <libgnomecanvas/libgnomecanvas.h>

#include "compat.hh"


/*
 * gtk_menu_item_set_label_markup:
 * an abstraction to let us set the menu label text.
 * Implementation given by John (J5) Palmieri
 * on the gnome-devel-list mailing list,
 * and direct access to the label suggested by Michael Meeks.
 * Thanks!
 */
void
gtk_menu_item_set_label_markup(GtkMenuItem *menu_item, const char *label)
{
	gtk_label_set_markup(GTK_LABEL(GTK_BIN(menu_item)->child), label);
}

/*
  redraw_if_visible and put_item_after ARE
 * COPIED VERBATIM libgnomecanvas-2.2.0.1's gnome-canvas.c.
 * just because they're not exported.
 * They should go away if canvas exports 
 * gnome_canvas_item_lower_below.
 */
static void
redraw_if_visible (GnomeCanvasItem *item)
{
	if (item->object.flags & GNOME_CANVAS_ITEM_VISIBLE)
		gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2 + 1, item->y2 + 1);
}
static gboolean
put_item_after (GList *link, GList *before)
{
	GnomeCanvasGroup *parent;

	if (link == before)
		return FALSE;

	parent = GNOME_CANVAS_GROUP (GNOME_CANVAS_ITEM (link->data)->parent);

	if (before == NULL) {
		if (link == parent->item_list)
			return FALSE;

		link->prev->next = link->next;

		if (link->next)
			link->next->prev = link->prev;
		else
			parent->item_list_end = link->prev;

		link->prev = before;
		link->next = parent->item_list;
		link->next->prev = link;
		parent->item_list = link;
	} else {
		if ((link == parent->item_list_end) && (before == parent->item_list_end->prev))
			return FALSE;

		if (link->next)
			link->next->prev = link->prev;

		if (link->prev)
			link->prev->next = link->next;
		else {
			parent->item_list = link->next;
			parent->item_list->prev = NULL;
		}

		link->prev = before;
		link->next = before->next;

		link->prev->next = link;

		if (link->next)
			link->next->prev = link;
		else
			parent->item_list_end = link;
	}
	return TRUE;
}

/**
 * gnome_canvas_item_lower_below:
 * @item: A canvas item whose stacking order should change.
 * @superior: The item that will be just above ITEM upon exit.
 *
 * Raises the item in the stacking order so that it just just below
 * superior.
 **/
void
gnome_canvas_item_lower_below(GnomeCanvasItem *item, GnomeCanvasItem *superior)
{
	GList *item_link, *superior_link;
	GnomeCanvasGroup *parent;

	g_return_if_fail(GNOME_IS_CANVAS_ITEM(item));
	g_return_if_fail(GNOME_IS_CANVAS_ITEM(superior));

	if (!item->parent)
		return;  /* not part of a canvas? */
	if (!superior->parent)
		return;  /* not part of a canvas? */
	g_assert(item->parent != superior->parent);  /* can't move between canvases */

	parent = GNOME_CANVAS_GROUP(item->parent);
	item_link = g_list_find(parent->item_list, item);
	g_assert(item_link != NULL);
	superior_link = g_list_find(parent->item_list, item);
	g_assert(superior_link != NULL);

	/* move it */
	if (put_item_after(item_link, superior_link)) {
		redraw_if_visible(item);
		item->canvas->need_repick = TRUE;
	};

}

#endif /* USE_GTK_BLOB */
